#!/usr/bin/env python

"""Get the distribution of classes in a dataset."""

# 3rd party module
import click


@click.command(name='distribution', help=__doc__)
@click.option('--gt', 'gt_filepath',
              required=True,
              type=click.Path(exists=True),
              help='List of labels for the dataset')
def cli(gt_filepath):
    """
    Command line interface to get the distribution of classes in a file.

    Parameters
    ----------
    gt_filepath : str
    """
    # Read text file
    with open(gt_filepath, 'r') as fp:
        read_lines = fp.readlines()
        labels = [line.rstrip('\n') for line in read_lines]

    distribution = get_distribution(labels)
    labels = sorted(distribution.items(),
                    key=lambda n: (-n[1], n[0]))
    label_len = max(len(label[0]) for label in labels)
    count_len = max(len(str(label[1])) for label in labels)
    total_count = sum(label[1] for label in labels)
    for label, count in labels:
        print('{percentage:5.2f}% {label:<{label_len}} '
              '({count:>{count_len}} elements)'
              .format(label=label,
                      count=count,
                      percentage=count / float(total_count) * 100.0,
                      label_len=label_len,
                      count_len=count_len))


def get_distribution(labels):
    """
    Get the distribution of the labels.

    Prameters
    ---------
    labels : list of str
        This list is non-unique.

    Returns
    -------
    distribution : dict
        Maps (label => count)

    Examples
    --------
    >>> dist = get_distribution(['de', 'de', 'en'])
    >>> sorted(dist.items())
    [('de', 2), ('en', 1)]
    """
    distribution = {}
    for label in labels:
        if label not in distribution:
            distribution[label] = 1
        else:
            distribution[label] += 1
    return distribution



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Calculate the confusion matrix (CSV inputs)."""

# core modules
import csv
import json
import logging

# 3rd party modules
import numpy as np
import click


@click.command(name='get-cm', help=__doc__)
@click.option('--predictions', 'cm_dump_filepath',
              required=True,
              type=click.Path(exists=True),
              help='CSV file with delimiter ;')
@click.option('--gt', 'gt_filepath',
              required=True,
              type=click.Path(exists=True),
              help='CSV file with delimiter ;')
@click.option('--n', 'n',
              required=True,
              type=int,
              help='Number of classes')
def main(cm_dump_filepath, gt_filepath, n):
    """
    Calculate a confusion matrix.

    Parameters
    ----------
    cm_dump_filepath : str
        CSV file with delimter ; and quoting char "
        The first field is an identifier, the second one is the index of the
        predicted label
    gt_filepath : str
        CSV file with delimter ; and quoting char "
        The first field is an identifier, the second one is the index of the
        ground truth
    n : int
        Number of classes
    """
    cm = calculate_cm(cm_dump_filepath, gt_filepath, n)
    # Write JSON file
    path = 'cm.json'
    with open(path, 'w') as outfile:
        str_ = json.dumps(cm.tolist(), indent=2,
                          separators=(',', ': '), ensure_ascii=False)
        outfile.write(str_)
    logging.info('cm was written to \'{}\''.format(path))


def calculate_cm(cm_dump_filepath, gt_filepath, n):
    """
    Calculate a confusion matrix.

    Parameters
    ----------
    cm_dump_filepath : str
        CSV file with delimter ; and quoting char "
        The first field is an identifier, the second one is the index of the
        predicted label
    gt_filepath : str
        CSV file with delimter ; and quoting char "
        The first field is an identifier, the second one is the index of the
        ground truth
    n : int
        Number of classes

    Returns
    -------
    confusion_matrix : numpy array (n x n)
    """
    cm = np.zeros((n, n), dtype=int)

    # Read CSV files
    with open(cm_dump_filepath, 'r') as fp:
        reader = csv.reader(fp, delimiter=';', quotechar='"')
        predictions = [row for row in reader]

    with open(gt_filepath, 'r') as fp:
        reader = csv.reader(fp, delimiter=';', quotechar='"')
        truths = [row for row in reader]

    ident2truth_index = {}
    for identifier, truth_index in truths:
        ident2truth_index[identifier] = int(truth_index)

    if len(predictions) != len(truths):
        msg = ('len(predictions) = {} != {} = len(truths)"'
               .format(len(predictions), len(truths)))
        raise ValueError(msg)

    for ident, pred_index in predictions:
        cm[ident2truth_index[ident]][int(pred_index)] += 1

    return cm



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Calculate the confusion matrix (one label per line)."""

# core modules
import csv
import json
import logging
import os
import sys

# 3rd party modules
import click
import numpy as np
import sklearn.metrics

# internal modules
import clana.utils

logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                    level=logging.DEBUG,
                    stream=sys.stdout)


@click.command(name='get-cm-simple', help=__doc__)
@click.option('--labels', 'label_filepath',
              required=True,
              type=click.Path(exists=True),
              help='CSV file with delimiter ;')
@click.option('--gt', 'gt_filepath',
              required=True,
              type=click.Path(exists=True),
              help='CSV file with delimiter ;')
@click.option('--predictions', 'predictions_filepath',
              required=True,
              type=click.Path(exists=True),
              help='CSV file with delimiter ;')
@click.option('--clean',
              default=False,
              is_flag=True,
              help='Remove classes that the classifier doesn\'t know')
def main(label_filepath, gt_filepath, predictions_filepath, clean):
    """
    Get a simple confunsion matrix.

    Parameters
    ----------
    label_filepath : str
        Path to a CSV file with delimiter ;
    gt_filepath : str
        Path to a CSV file with delimiter ;
    predictions : str
        Path to a CSV file with delimiter ;
    clean : bool, optional (default: False)
        Remove classes that the classifier doesn't know
    """
    label_filepath = os.path.abspath(label_filepath)
    labels = clana.utils.load_labels(label_filepath, 0)

    # Read CSV files
    with open(gt_filepath, 'r') as fp:
        reader = csv.reader(fp, delimiter=';', quotechar='"')
        truths = [row[0] for row in reader]

    with open(predictions_filepath, 'r') as fp:
        reader = csv.reader(fp, delimiter=';', quotechar='"')
        predictions = [row[0] for row in reader]

    cm = calculate_cm(labels, truths, predictions, clean=False)
    # Write JSON file
    cm_filepath = os.path.abspath('cm.json')
    logging.info("Write results to '{}'.".format(cm_filepath))
    with open(cm_filepath, 'w') as outfile:
        str_ = json.dumps(cm.tolist(), indent=2,
                          separators=(',', ': '), ensure_ascii=False)
        outfile.write(str_)
    print(cm)


def calculate_cm(labels,
                 truths,
                 predictions,
                 replace_unk_preds=False,
                 clean=False):
    """
    Calculate a confusion matrix.

    Parameters
    ----------
    labels : list
    truths : list
    predictions : list
    replace_unk_preds : bool, optional (default: True)
        If a prediction is not in the labels in label_filepath, replace it
        with UNK
    clean : bool, optional (default: False)
        Remove classes that the classifier doesn't know

    Returns
    -------
    confusion_matrix : numpy array (n x n)
    """
    # Check data
    if len(predictions) != len(truths):
        msg = ('len(predictions) = {} != {} = len(truths)"'
               .format(len(predictions), len(truths)))
        raise ValueError(msg)

    label2i = {}  # map a label to 0, ..., n
    for i, label in enumerate(labels):
        label2i[label] = i

    if clean:
        logging.debug('@' * 80)
        preds = []
        truths_tmp = []
        for tru, pred in zip(truths, predictions):
            if tru in predictions:
                truths_tmp.append(tru)
                preds.append(pred)
        predictions = preds
        truths = truths_tmp

    if replace_unk_preds:
        preds = []
        for pred in predictions:
            if label in label2i:
                preds.append(label)
            else:
                preds.append('UNK')
        predictions = preds

    # Sanity check
    for label in truths:
        if label not in label2i:
            logging.error('Could not find label \'{}\''.format(label))
            sys.exit(-1)

    n = len(labels)
    for label in predictions:
        if label not in label2i:
            label2i[label] = len(labels)
            n = len(labels) + 1
            logging.error('Could not find label \'{}\' in labels file => '
                          'Add class UNK'
                          .format(label))

    # TODO: do no always filter
    filter_data_unk = True
    if filter_data_unk:
        truths2, predictions2 = [], []
        for tru, pred in zip(truths, predictions):
            if pred != 'unk':  # TODO: tru != 'UNK'!!!
                truths2.append(tru)
                predictions2.append(pred)
        truths = truths2
        predictions = predictions2

    report = sklearn.metrics.classification_report(truths, predictions,
                                                   labels=labels)
    print(report)
    print("Accuracy: {:.2f}%"
          .format(sklearn.metrics.accuracy_score(truths, predictions) * 100))

    cm = np.zeros((n, n), dtype=int)

    for truth_label, pred_label in zip(truths, predictions):
        cm[label2i[truth_label]][label2i[pred_label]] += 1

    return cm



"""CLANA is a toolkit for classifier analysis."""

from setuptools import find_packages
from setuptools import setup
import io
import os

here = os.path.abspath(os.path.dirname(__file__))
try:
    # obtain long description from README and CHANGES
    # Specify encoding to get a unicode type in Python 2 and a str in Python 3
    with io.open(os.path.join(here, 'README.md'), 'r', encoding='utf-8') as f:
        README = f.read()
except IOError:
    README = ''

config = {
    'name': 'clana',
    'version': '0.2.0',
    'author': 'Martin Thoma',
    'author_email': 'info@martin-thoma.de',
    'maintainer': 'Martin Thoma',
    'maintainer_email': 'info@martin-thoma.de',
    'packages': find_packages(),
    'scripts': ['bin/clana'],
    'platforms': ['Linux'],
    'url': 'https://github.com/MartinThoma/clana',
    'license': 'MIT',
    'description': __doc__,
    'long_description': README,
    'install_requires': [
        'click>=6.7',
        'matplotlib>=2.1.1',
        'numpy>=1.14.0',
        'PyYAML>=3.12',
        'scikit-learn>=0.19.1',
        'scipy>=1.0.0',
    ],
    'keywords': ['Machine Learning', 'Data Science'],
    'download_url': 'https://github.com/MartinThoma/language-identification',
    'classifiers': ['Development Status :: 2 - Pre-Alpha',
                    'Environment :: Console',
                    'Intended Audience :: Developers',
                    'Intended Audience :: Science/Research',
                    'Intended Audience :: Information Technology',
                    'License :: OSI Approved :: MIT License',
                    'Natural Language :: English',
                    'Programming Language :: Python :: 3.5',
                    'Topic :: Scientific/Engineering :: Information Analysis',
                    'Topic :: Software Development',
                    'Topic :: Utilities'],
    'zip_safe': False,
}

setup(**config)
